@@ -45,3 +45,13 @@ def info():
         except ImportError:
             print('%10s %10s Not Found' % (modui, ''))
     print()
+
+    # reference
+    print("""Reference
+------------
+Please cite:
+Yuzhi Zhang, Haidi Wang, Weijie Chen, Jinzhe Zeng, Linfeng Zhang, Han Wang, and Weinan E,
+DP-GEN: A concurrent learning platform for the generation of reliable deep learning
+based potential energy models, Computer Physics Communications, 2020, 107206.
+------------
+""")
